
// **************************************
// Header Files
// **************************************

#include "driveroptionlist.h"

// **************************
// Class Declaration
// **************************

DriverOptionList::DriverOptionList(void)
{  // constructor
  size=0;
  lineNr=0;
  dList=NULL;
}

DriverOptionList::~DriverOptionList(void)
{  // destructor
  if (dList) delete [] dList;
  dList=NULL;
}

DriverOptionList::DriverOptionList(const DriverOptionList &dis)
{  // copy constructor

  lineNr=0;

  size=dis.GetSize();
  dList=new DriverOption[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }
}

DriverOptionList& DriverOptionList::operator=(const DriverOptionList &dis)
{  // copy constructor

  Clear();

  size=dis.GetSize();
  dList=new DriverOption[size];

  for (int i=0; i<size;i++)
  {
    dList[i]=dis[i];
  }

  return(*this);
}

void DriverOptionList::Remove(const int index)
{  // remove a DriverOption

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("DriverOptionList::Remove"), wxICON_ERROR);
    return;
  }

  int i;  // loop counter

  if (size>1)
  {
    DriverOption *dis=new DriverOption[size-1];

    for (i=0; i<index; i++) dis[i]=dList[i];        // copy old entries
    for (i=index+1; i<size; i++) dis[i-1]=dList[i]; // and ignore index

    if (dList) delete [] dList;   // delete old list
    dList=dis;         // set pointer
    dis=NULL;
  }
  else
  {
    if (dList) delete [] dList;   // delete old list
    dList=NULL;         // set pointer
  }

  size--;              // set size
}

void DriverOptionList::Add(const DriverOption& dis)
{  // add an DriverOption and value

  DriverOption *dL=new DriverOption[size+1];

  for (int i=0; i<size; i++) dL[i]=dList[i]; // copy old entries

  dL[size]=dis;   // set las entry

  if (dList) delete [] dList;   // delete old list
  dList=dL;         // set pointer
  dL=NULL;

  size++;
}

void DriverOptionList::Clear(void)
{    // clear everything

  if (dList) delete [] dList;
  dList=NULL;

  size=0;
  lineNr=0;
}

const DriverOption& DriverOptionList::operator[](const int index) const
{  // get single DriverOption

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("DriverOptionList::Get"), wxICON_ERROR);
  }

  return(dList[index]);
}

DriverOption& DriverOptionList::operator[](const int index)
{  // get single device

  if (index>=size)
  {
    wxMessageBox(_("Try to access item outside boundaries!"), _T("DriverOptionList::operator[]"), wxICON_ERROR);
    return(dList[index]);  // return empty device
  }

  return(dList[index]);
}

const int DriverOptionList::GetSize(void) const
{  // return size

  return(size);

}

// special functions

void DriverOptionList::StripSpaces(wxString& line) const
{         // convert tabs to spaces, delete spaces from left and right,

  line.Replace(_T("\t"), _T(" "));         // replace tabs with spaces
  while (line.Replace(_T("  "), _T(" "))>0);  // replace double spaces with single space
  line.Trim(true);     // strip spaces from right
  line.Trim(false);    // strip spaces from left
}

void DriverOptionList::StripRight(wxString& line, const int num) const
{  // delete n right characters

  line=line.Right(line.Length()-num);
  StripSpaces(line);
}

const wxString DriverOptionList::GetNextLine(wxTextFile& file)
{
  wxString line;

  if (lineNr==0)         // handle first line separatly
  {
    line=file.GetFirstLine();  // read first line
    lineNr++;
  }
  else
  {
    line=file.GetNextLine();    // read next line
    lineNr++;
  }
  return(line);
}

const bool DriverOptionList::ShowError(const optionErrorID error, const wxString& filename, const wxString& line)
{   // show error message when reading file, return true if the user wants to stop loading

  wxString xLine(line);
  StripSpaces(xLine);     // strip spaces and comments

  wxString message;
  message << _("Error in file \"") << filename;
  message << _("\" in line number ") << lineNr << _T(": \n\"") << xLine << _T("\"");
  message << _T(".\n\n");

  switch (error)
  {
    case NOOPTION:
      message << _("Argument outside <option>-tag.");
      break;
    case TWOOPTIONS:
      message << _("Second <option>-tag found without closing </option> before.");
      break;
    case NOOPTIONEND:
      message << _("Option line does not end with '>'.");
      break;
    case NODEFAULTEND:
      message << _("</default> missing at end of line.");
      break;
    case NOARGUMENTEND:
      message << _("</argument> missing at end of line.");
      break;
    case NAMEERROR:
      message << _("Error in name assignment.");
      break;
    case VALUEERROR:
      message << _("Error in value assignment.");
      break;
    case VALUEVALUE:
      message << _("Error in value assignment: Type must be 'string', 'integer' or 'boolean'.");
      break;
    case COMMENTERROR:
      message << _("Error in comment assignment.");
      break;
    case STRICTERROR:
      message << _("Error in strict assignment.");
      break;
    case STRICTVALUE:
      message << _("Error in strict assignment: Type must be 'true' or 'false'.");
      break;
    case CLOSEOPTION:
      message << _("</option> missing at end of file.");
      break;
    case TWODEFAULT:
      message << _("More than one <default>-tag in one option.");
      break;
    case NAMEMISSING:
      message << _("Option name is missing.");
      break;
    case EMPTYARGUMENT2:
      message << _("<argument> is empty.");
      break;
    case EMPTYDEFAULT:
      message << _("<default> is empty.");
      break;
    default:
      message << _("Unrecognized line.");
      break;
  } // switch

  if (error!=NAMEMISSING)
  {
    message << _T("\n\n") << _("Should I ignore this and go on?") << _("\n(Warning: It is not sure what happens next!)");

    if (wxMessageBox(message, _("Error: Reading file"), wxICON_ERROR | wxYES_NO | wxYES_DEFAULT )==wxNO)
    {
      return(true);
    }
    else return(false);
  }
  else
  {
    wxMessageBox(message, _("Error: Reading file"), wxICON_ERROR);
    return(true);
  }
  return(false);
}

const bool DriverOptionList::Load(const wxString& filename)
{  // load driver option lists

  wxTextFile file(filename);  // create text file handler
  wxString line;
  wxString orgLine;
  bool comment=false;         // flag if a comment has started
  bool option=false;
  DriverOption tempOption;
  optionErrorID error=UNKNOWNL;            // unknown error

  Clear();   // clear old before

  if (!file.Exists()) return(false);  // no option list defined yet

  //open file
  file.Open();                            // open file
  lineNr=0;                  // line number

  do
  {
    orgLine=GetNextLine(file);    // read next line
    line=orgLine;
    StripSpaces(line);

    if (!line.IsEmpty())  // line is not empty and no comment
    {
      if (line.StartsWith(_T("-->"))) // end comment
      {
        comment=false;
      }
      else if (!comment)
      {
        if (line.StartsWith(_T("<!--")))  // search for comment
        {
          comment=true;
        }
        else if (line.StartsWith(_T("<option")))
        {  // option begins

          if (option)
          {  // error
            tempOption.Clear();
            error=TWOOPTIONS;
            if (ShowError(error, filename, orgLine))
            {
              Clear();
              return(true);
            }
          }
          else
          {
            if (line.Last()=='>')
            {
              StripRight(line, 7);
              option=true;

              wxString xLine;
              int pos=-1;

              // search name
              xLine=line;
              pos=xLine.Find(_T("name=\""));
              if (pos>=0)
              {
                StripRight(xLine,pos+6);
                pos=xLine.Find('"');
                if (pos>=1)
                {
                  tempOption.SetName(xLine.Left(pos));
                }
                else
                {  // error
                  error=NAMEERROR;
                  if (ShowError(error, filename, orgLine))
                  {
                    Clear();
                    return(true);
                  }
                }
              }
              else
              {  // error, ignore
                error=NAMEMISSING;
                if (ShowError(error, filename, orgLine))
                {
                  Clear();
                  return(true);
                }
              }

              // search value
              xLine=line;
              pos=xLine.Find(_T("value=\""));
              if (pos>=0)
              {
                StripRight(xLine,pos+7);
                pos=xLine.Find('"');
                if (pos>=1)
                {
                  xLine=xLine.Left(pos);
                  if (xLine==_T("string") || xLine==_T("integer") || xLine==_T("boolean"))
                  {
                    tempOption.SetValue(xLine.Left(pos));
                  }
                  else
                  {  // error
                    error=VALUEVALUE;
                    if (ShowError(error, filename, orgLine))
                    {
                      Clear();
                      return(true);
                    }
                  }
                }
                else
                {  // error
                  error=VALUEERROR;
                  if (ShowError(error, filename, orgLine))
                  {
                    Clear();
                    return(true);
                  }
                }
              }

              // search comment
              xLine=line;
              pos=xLine.Find(_T("comment=\""));
              if (pos>=0)
              {
                StripRight(xLine,pos+9);
                pos=xLine.Find('"');
                if (pos>=1)
                {
                  tempOption.SetComment(xLine.Left(pos));
                }
                else
                {  // error
                  error=COMMENTERROR;
                  if (ShowError(error, filename, orgLine))
                  {
                    Clear();
                    return(true);
                  }
                }
              }

              // search strict
              xLine=line;
              pos=xLine.Find(_T("strict=\""));
              if (pos>=0)
              {
                StripRight(xLine,pos+8);
                pos=xLine.Find('"');
                if (pos>=1)
                {
                  xLine=xLine.Left(pos);
                  if (xLine==_T("true"))
                  {
                    tempOption.SetStrict(true);
                  }
                  else if (xLine==_T("false"))
                  {
                    tempOption.SetStrict(false);
                  }
                  else
                  {  // error
                    error=STRICTVALUE;
                    if (ShowError(error, filename, orgLine))
                    {
                      Clear();
                      return(true);
                    }
                  }
                }
                else
                {  // error
                  error=STRICTERROR;
                  if (ShowError(error, filename, orgLine))
                  {
                    Clear();
                    return(true);
                  }
                }
              }

            }
            else
            {  // error
              error=NOOPTIONEND;
              if (ShowError(error, filename, orgLine))
              {
                Clear();
                return(true);
              }
            }
          }
        }
        else if (line.StartsWith(_T("</option>")))
        {  // option ends

          Add(tempOption);     // add option to list
          tempOption.Clear();  // clear option again
          option=false;
        }
        else if (line.StartsWith(_T("<default>")))
        {  // default argument

          if (option)  // option is on
          {
            if (!tempOption.GetDefault().IsEmpty())
            {
              error=TWODEFAULT;
              if (ShowError(error, filename, orgLine))
              {
                Clear();
                return(true);
              }
            }

            StripRight(line, 9);
            int pos=line.Find(_T("</default>"));
            if (pos>=0)
            {
              line=line.Left(line.Len()-10);
              StripSpaces(line);

              if (line.IsEmpty())
              {
                error=EMPTYDEFAULT;
                if (ShowError(error, filename, orgLine))
                {
                  Clear();
                  return(true);
                }
              }
              else tempOption.SetDefault(line);
            }
            else
            { // error
              error=NODEFAULTEND;
              if (ShowError(error, filename, orgLine))
              {
                Clear();
                return(true);
              }
            }
          }
          else
          {  // error
            error=NOOPTION;
            if (ShowError(error, filename, orgLine))
            {
              Clear();
              return(true);
            }
          }
        }
        else if (line.StartsWith(_T("<argument>")))
        {  // default argument

          if (option)  // option is on
          {
            StripRight(line, 10);
            int pos=line.Find(_T("</argument>"));
            if (pos>=0)
            {
              line=line.Left(line.Len()-11);
              StripSpaces(line);

              if (line.IsEmpty())
              {
                error=EMPTYARGUMENT2;
                if (ShowError(error, filename, orgLine))
                {
                  Clear();
                  return(true);
                }
              }
              else tempOption.AddArgument(line);
            }
            else
            { // error, ignore
              error=NOARGUMENTEND;
              if (ShowError(error, filename, orgLine))
              {
                Clear();
                return(true);
              }
            }
          }
          else
          {  // error, ignore
            error=NOOPTION;
            if (ShowError(error, filename, orgLine))
            {
              Clear();
              return(true);
            }
          }
        }
        else  // unrecognized entry
        {
          error=UNKNOWNL;
          if (ShowError(error, filename, orgLine))
          {
            Clear();
            return(true);
          }
        }
      }
    } // line is not empty

  } while (!file.Eof());                  // repeat until file end

  if (option)
  {
    error=CLOSEOPTION;
    if (ShowError(error, filename, orgLine))
    {
      Clear();
      return(true);
    }
  }


  return(false);
}

const int DriverOptionList::Find(const wxString& name) const
{ // search option

  int pos=-1;

  for (int i=0; i<size; i++)
  {
    if (dList[i].GetName()==name)
    {
      pos=i;
      break;
    }
  }

  return(pos);
}


