// **************************************
// Header Files
// **************************************

#include "filehandler.h"

// **************************
// Class Declaration
// **************************

FileHandler::FileHandler(void)
{
  lineNr=0;
}

FileHandler::~FileHandler(void)
{}

void FileHandler::StripSpaces(wxString& line) const
{         // convert tabs to spaces, delete spaces from left and right,

  line.Replace(_T("\t"), _T(" "));         // replace tabs with spaces
  while (line.Replace(_T("  "), _T(" "))>0);  // replace double spaces with single space
  line.Trim(true);     // strip spaces from right
  line.Trim(false);    // strip spaces from left
}

void FileHandler::StripRight(wxString& line, const int num) const
{  // delete n right characters

  line=line.Right(line.Length()-num);
  StripSpaces(line);
}

const wxString FileHandler::GetNextLine(wxTextFile& file)
{
  wxString line;

  if (lineNr==0)         // handle first line separatly
  {
    line=file.GetFirstLine();  // read first line
    lineNr++;
  }
  else
  {
    line=file.GetNextLine();    // read next line
    lineNr++;
  }

  return(line);
}

void FileHandler::LoadSettings(Settings& settings)
{  // load settings in ~/.xorg-edit

  wxTextFile file(settings.GetFilename());    // create text file handler
  wxString line;

  if (!file.Exists())
  {
    SaveSettings(settings, true);
    return;
  }

  file.Open();    // open file
  lineNr=0;       // set line number to start

  do
  {
    line=GetNextLine(file);       // read first line
    StripSpaces(line);  // delete spaces around

    if (!line.IsEmpty())  // line is not empty
    {
      if (line.StartsWith(_T("LANGUAGE=")))
      {
        StripRight(line, 9);
        long lang;
        line.ToLong(&lang);  // convert to language ID
        settings.SetLanguage(lang);
      }
      else if (line.StartsWith(_T("OPTDIR=\"")))
      {
        StripRight(line, 8);
        if (line.Last()=='"')
        {
          line.RemoveLast();

          if (wxDirExists(line)) // option directory does exists
          {
            settings.SetOptionDir(line);
          }
          else
          {  // fallback mode

            settings.SetOptionDir(settings.GetProgDir()+_T("/options"));

            if (!wxDirExists(settings.GetOptionDir()))  // option directory does not exists
            {  // another fallback
              settings.SetOptionDir(wxGetCwd());
            }
          }

        }
      }
      else if (line.StartsWith(_T("LASTDIR=\"")))
      {
        StripRight(line, 9);
        if (line.Last()=='"')
        {
          line.RemoveLast();
          settings.SetCurrentDir(line);
        }
      }
      else if (line.StartsWith(_T("LASTFILE=\"")))
      {
        StripRight(line, 10);
        if (line.Last()=='"')
        {
          line.RemoveLast();
          settings.SetCurrentFilename(line);
        }
      }
      else if (line.StartsWith(_T("LOADLAST=")))
      {
        StripRight(line, 9);
        if (line.Len()==1)
        {
          if (line==_T("0"))
          {
            settings.SetLoadLastFile(false);
          }
          else
          {
            settings.SetLoadLastFile(true);
          }
        }
      }
      else if (line.StartsWith(_T("SAVEPROFILE=")))
      {
        StripRight(line, 12);
        if (line.Len()==1)
        {
          if (line==_T("0"))
          {
            settings.SetSaveSettings(false);
          }
          else
          {
            settings.SetSaveSettings(true);
          }
        }
      }
      else if (line.StartsWith(_T("SHOWSPLASH=")))
      {
        StripRight(line, 11);

        if (line.Len()==1)
        {
          if (line==_T("0"))
          {
            settings.SetShowSplash(false);
          }
          else
          {
            settings.SetShowSplash(true);
          }
        }
      }
      else if (line.StartsWith(_T("SHOWBLANKTEST=")))
      {
        StripRight(line, 14);

        if (line.Len()==1)
        {
          if (line==_T("0"))
          {
            settings.SetShowBlank(false);
          }
          else
          {
            settings.SetShowBlank(true);
          }
        }
      }
      else if (line.StartsWith(_T("TESTBEFORESAVE=")))
      {
        StripRight(line, 15);

        if (line.Len()==1)
        {
          if (line==_T("0"))
          {
            settings.SetRememberTest(false);
          }
          else
          {
            settings.SetRememberTest(true);
          }
        }
      }
      else
      {
          // unrecognized line... ignore
      }
    } // line is not empty

  } while (!file.Eof());                  // repeat until file end

}

void FileHandler::SaveSettings(const Settings& settings, const bool start)
{  // save settings in ~/.xorg-edit

  // save following things
  // - language
  // - option directory
  // - last working directory
  // - last edited file
  // - always load last
  // - save profile at end
  // - show splash screen

  wxTextFile file(settings.GetFilename());    // create text file handler
  wxString tempStr;

  // open file
  if (file.Exists())
  {
    file.Open();                         // open file
    file.Clear();
  }
  else
  {
    file.Create();    // create file
  }

  // save language
  tempStr.Printf(_T("LANGUAGE=%d"),settings.GetLanguage());
  file.AddLine(tempStr);

  // save option directory
  tempStr=_T("OPTDIR=\"");
  tempStr+=settings.GetOptionDir();
  tempStr+=_T("\"");
  file.AddLine(tempStr);

  // save last working directory
  tempStr=_T("LASTDIR=\"");
  tempStr+=settings.GetCurrentDir();
  tempStr+=_T("\"");
  file.AddLine(tempStr);

  // save last file
  tempStr=_T("LASTFILE=\"");
  tempStr+=settings.GetCurrentFilename();
  tempStr+=_T("\"");
  file.AddLine(tempStr);

  // save flag if last file should be loaded
  tempStr=_T("LOADLAST=");
  if (settings.GetLoadLastFile()) tempStr+=_T("1");
  else tempStr+=_T("0");
  file.AddLine(tempStr);

  // save flag if profle should be saved at the end
  tempStr=_T("SAVEPROFILE=");
  if (settings.GetSaveSettings()) tempStr+=_T("1");
  else tempStr+=_T("0");
  file.AddLine(tempStr);

  // save flag if warning should be shown at start
  tempStr=_T("SHOWSPLASH=");
  if (settings.GetShowSplash()) tempStr+=_T("1");
  else tempStr+=_T("0");
  file.AddLine(tempStr);

  tempStr=_T("SHOWBLANKTEST=");
  if (settings.GetShowBlank()) tempStr+=_T("1");
  else tempStr+=_T("0");
  file.AddLine(tempStr);

  tempStr=_T("TESTBEFORESAVE=");
  if (settings.GetRememberTest()) tempStr+=_T("1");
  else tempStr+=_T("0");
  file.AddLine(tempStr);

  // file.AddLine(_T(""));  // empty line

  // save all changes
  if (!file.Write() && start==false)
  {
    wxString tempStr;
    tempStr << _("Could not save profile to ") << settings.GetFilename() << _T(".\n");
    wxMessageBox(tempStr, _("Error: Saving profile"), wxICON_ERROR);
  }

}

void FileHandler::LoadDrivers(wxArrayString& list, const wxString& filename)
{  // load driver lists

  wxTextFile file(filename);    // create text file handler
  wxString line;

  if (file.Exists())
  {
    list.Clear();  // clear list

    file.Open();                            // open file
    lineNr=0;

    do
    {
      line=GetNextLine(file);    // read next line
      StripSpaces(line);  // delete spaces around

      if (!line.IsEmpty())  // line is not empty
      {
       if (list.Index(line)==wxNOT_FOUND)
       {  // only add if not in list
        list.Add(line);
       }
      } // line is not empty

    } while (!file.Eof());                  // repeat until file end
  }
}
