#ifndef XORGFILE_H
#define XORGFILE_H

// **************************************
// Header Files
// **************************************

#include "global.h"
#include "devicelist.h"
#include "extensionlist.h"
#include "filelist.h"
#include "inputdevicelist.h"
#include "modulelist.h"
#include "monitorlist.h"
#include "screenlist.h"
#include "serverflaglist.h"
#include "serverlayoutlist.h"
#include "drimode.h"
#include "option.h"
#include <wx/textfile.h>

// **************************
// Constants
// **************************

enum xorgSectionID
{ // IDs for sections in xorg.conf

  FILES=100,
  SERVERFLAGS,
  MODULE,
  INPUTDEVICE,
  DEVICE,
  VIDEOADAPTOR,    // not recognized yet (and ever)
  MONITOR,
  SCREEN,
  SERVERLAYOUT,
  DRI,
  VENDOR,          // not recognized yet (and ever?)
  EXTENSIONS,
  DISPLAY,
  NOSEC
};

enum xorgErrorID
{ // IDs for errors while reading xorg.conf
  UNKNOWN=200,
  NOERROR,
  NOSUBSECTION,
  NOSECTION,
  TWOSECTIONS,
  NOCDEVICE,
  NOCSCREEN,
  NOCMONITOR,
  NOCIDEVICE,
  NOMONITORS,
  NODEVICES,
  NOIDEVICES,
  NOSCREENS,
  NOSLAYOUTS,
  EMPTYARGUMENT,
  NODEVICEID,
  NODEVICEDRIVER,
  SAMEDEVICEID,
  NOMONITORID,
  SAMEMONITORID,
  NOIDEVICEID,
  SAMEIDEVICEID,
  NOIDEVICEDRIVER,
  NOSCREENID,
  NOSLAYOUTID,
  NOMODUL,
  SECDRI
};

// **************************
// Class Definition
// **************************

class XorgFile
{
public:
  XorgFile(void);
  XorgFile(const XorgFile& xorg);
  virtual ~XorgFile();
  XorgFile& operator=(const XorgFile& xorg);

  const FileList& Files(void) const;
  const ModuleList& Modules(void) const;
  const ServerLayoutList& ServerLayouts(void) const;
  const InputDeviceList& InputDevices(void) const;
  const DeviceList& Devices(void) const;
  const MonitorList& Monitors(void) const;
  const ScreenList& Screens(void) const;
  const ServerFlagList& ServerFlags(void) const;
  const ExtensionList& Extensions(void) const;
  const DriMode& Dri(void) const;

  FileList& Files(void);
  ModuleList& Modules(void);
  ServerLayoutList& ServerLayouts(void);
  InputDeviceList& InputDevices(void);
  DeviceList& Devices(void);
  MonitorList& Monitors(void);
  ScreenList& Screens(void);
  ServerFlagList& ServerFlags(void);
  ExtensionList& Extensions(void);
  DriMode& Dri(void);

  void AddComment(const wxString& com);
  const wxArrayString& GetComment(void) const;

  void Clear(void);

  const bool Load(const wxString& name);
  const bool Save(const wxString& name, const wxString& version);
  void Preview(const wxString& name, const wxString& version) const;

private:
  const wxString GetNextLine(wxTextFile& file);
  void StripSpaces(wxString& line) const;         // convert tabs to spaces, delete spaces from left and right
  void StripRight(wxString& line, const int num) const;    // delete n right characters

  const bool GetArgument(wxString &arg, const wxString &line) const;
    // get a line and extract the last argument

  const bool GetOption(Option& opt, const wxString &line) const;
    // get a line and extract option and argument

  const bool FreeComments(wxString& line, wxString& comment) const;    // delete comments in line, return true if line was only comment

  const bool ShowError(const xorgErrorID error, const xorgSectionID section, const wxString& filename, const wxString& line);
    // show error message when reading file, return true if the user wants to stop loading

  int lineNr; // line counter
  wxString filename;

  FileList files;
  ModuleList modules;
  ServerLayoutList serverLayouts;
  InputDeviceList inputDevices;
  DeviceList devices;
  MonitorList monitors;
  ScreenList screens;
  ServerFlagList serverFlags;
  ExtensionList extensions;
  DriMode driEntry;
  wxArrayString comment;


};

#endif // XORGFILE_H
